@@ -1,3 +1,5 @@
+"""TVAESynthesizer module."""
+
 import numpy as np
 import torch
 from torch.nn import Linear, Module, Parameter, ReLU, Sequential
@@ -6,10 +8,21 @@ from torch.optim import Adam
 from torch.utils.data import DataLoader, TensorDataset
 
 from ctgan.data_transformer import DataTransformer
-from ctgan.synthesizers.base import BaseSynthesizer
+from ctgan.synthesizers.base import BaseSynthesizer, random_state
 
 
 class Encoder(Module):
+    """Encoder for the TVAESynthesizer.
+
+    Args:
+        data_dim (int):
+            Dimensions of the data.
+        compress_dims (tuple or list of ints):
+            Size of each hidden layer.
+        embedding_dim (int):
+            Size of the output vector.
+    """
+
     def __init__(self, data_dim, compress_dims, embedding_dim):
         super(Encoder, self).__init__()
         dim = data_dim
@@ -20,12 +33,14 @@ class Encoder(Module):
                 ReLU()
             ]
             dim = item
+
         self.seq = Sequential(*seq)
         self.fc1 = Linear(dim, embedding_dim)
         self.fc2 = Linear(dim, embedding_dim)
 
-    def forward(self, input):
-        feature = self.seq(input)
+    def forward(self, input_):
+        """Encode the passed `input_`."""
+        feature = self.seq(input_)
         mu = self.fc1(feature)
         logvar = self.fc2(feature)
         std = torch.exp(0.5 * logvar)
@@ -33,6 +48,17 @@ class Encoder(Module):
 
 
 class Decoder(Module):
+    """Decoder for the TVAESynthesizer.
+
+    Args:
+        embedding_dim (int):
+            Size of the input vector.
+        decompress_dims (tuple or list of ints):
+            Size of each hidden layer.
+        data_dim (int):
+            Dimensions of the data.
+    """
+
     def __init__(self, embedding_dim, decompress_dims, data_dim):
         super(Decoder, self).__init__()
         dim = embedding_dim
@@ -45,19 +71,21 @@ class Decoder(Module):
         self.seq = Sequential(*seq)
         self.sigma = Parameter(torch.ones(data_dim) * 0.1)
 
-    def forward(self, input):
-        return self.seq(input), self.sigma
+    def forward(self, input_):
+        """Decode the passed `input_`."""
+        return self.seq(input_), self.sigma
 
 
-def loss_function(recon_x, x, sigmas, mu, logvar, output_info, factor):
+def _loss_function(recon_x, x, sigmas, mu, logvar, output_info, factor):
     st = 0
     loss = []
     for column_info in output_info:
         for span_info in column_info:
-            if len(column_info) != 1 or span_info.activation_fn != "softmax":
+            if span_info.activation_fn != 'softmax':
                 ed = st + span_info.dim
                 std = sigmas[st]
-                loss.append(((x[:, st] - torch.tanh(recon_x[:, st])) ** 2 / 2 / (std ** 2)).sum())
+                eq = x[:, st] - torch.tanh(recon_x[:, st])
+                loss.append((eq ** 2 / 2 / (std ** 2)).sum())
                 loss.append(torch.log(std) * x.size()[0])
                 st = ed
 
@@ -68,7 +96,7 @@ def loss_function(recon_x, x, sigmas, mu, logvar, output_info, factor):
                 st = ed
 
     assert st == recon_x.size()[1]
-    KLD = -0.5 * torch.sum(1 + logvar - mu.pow(2) - logvar.exp())
+    KLD = -0.5 * torch.sum(1 + logvar - mu**2 - logvar.exp())
     return sum(loss) * factor / x.size()[0], KLD / x.size()[0]
 
 
@@ -105,7 +133,19 @@ class TVAESynthesizer(BaseSynthesizer):
 
         self._device = torch.device(device)
 
-    def fit(self, train_data, discrete_columns=tuple()):
+    @random_state
+    def fit(self, train_data, discrete_columns=()):
+        """Fit the TVAE Synthesizer models to the training data.
+
+        Args:
+            train_data (numpy.ndarray or pandas.DataFrame):
+                Training Data. It must be a 2-dimensional numpy array or a pandas.DataFrame.
+            discrete_columns (list-like):
+                List of discrete columns to be used to generate the Conditional
+                Vector. If ``train_data`` is a Numpy array, this list should
+                contain the integer indices of the columns. Otherwise, if it is
+                a ``pandas.DataFrame``, this list should contain the column names.
+        """
         self.transformer = DataTransformer()
         self.transformer.fit(train_data, discrete_columns)
         train_data = self.transformer.transform(train_data)
@@ -114,7 +154,7 @@ class TVAESynthesizer(BaseSynthesizer):
 
         data_dim = self.transformer.output_dimensions
         encoder = Encoder(data_dim, self.compress_dims, self.embedding_dim).to(self._device)
-        self.decoder = Decoder(self.embedding_dim, self.compress_dims, data_dim).to(self._device)
+        self.decoder = Decoder(self.embedding_dim, self.decompress_dims, data_dim).to(self._device)
         optimizerAE = Adam(
             list(encoder.parameters()) + list(self.decoder.parameters()),
             weight_decay=self.l2scale)
@@ -127,7 +167,7 @@ class TVAESynthesizer(BaseSynthesizer):
                 eps = torch.randn_like(std)
                 emb = eps * std + mu
                 rec, sigmas = self.decoder(emb)
-                loss_1, loss_2 = loss_function(
+                loss_1, loss_2 = _loss_function(
                     rec, real, sigmas, mu, logvar,
                     self.transformer.output_info_list, self.loss_factor
                 )
@@ -136,7 +176,17 @@ class TVAESynthesizer(BaseSynthesizer):
                 optimizerAE.step()
                 self.decoder.sigma.data.clamp_(0.01, 1.0)
 
+    @random_state
     def sample(self, samples):
+        """Sample data similar to the training data.
+
+        Args:
+            samples (int):
+                Number of rows to sample.
+
+        Returns:
+            numpy.ndarray or pandas.DataFrame
+        """
         self.decoder.eval()
 
         steps = samples // self.batch_size + 1
@@ -154,5 +204,6 @@ class TVAESynthesizer(BaseSynthesizer):
         return self.transformer.inverse_transform(data, sigmas.detach().cpu().numpy())
 
     def set_device(self, device):
+        """Set the `device` to be used ('GPU' or 'CPU)."""
         self._device = device
         self.decoder.to(self._device)
